==> ideaIC-191.5701.16-sources.jar: com/android/tools/profilers/memory/adapters/LegacyAllocationCaptureObject.java <==
/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.profilers.memory.adapters;

import com.android.tools.adtui.model.Range;
import com.android.tools.profiler.proto.Common;
import com.android.tools.profiler.proto.MemoryProfiler;
import com.android.tools.profiler.proto.MemoryProfiler.LegacyAllocationEvent;
import com.android.tools.profiler.proto.MemoryServiceGrpc.MemoryServiceBlockingStub;
import com.android.tools.profilers.analytics.FeatureTracker;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.OutputStream;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.android.tools.profilers.memory.MemoryProfiler.saveLegacyAllocationToFile;

public final class LegacyAllocationCaptureObject implements CaptureObject {
  static final int DEFAULT_HEAP_ID = 0;
  static final String DEFAULT_HEAP_NAME = "default";
  static final long DEFAULT_CLASSLOADER_ID = -1;

  @NotNull private final MemoryServiceBlockingStub myClient;
  @NotNull private final ClassDb myClassDb;
  @NotNull private final Common.Session mySession;
  @NotNull private final MemoryProfiler.AllocationsInfo myInfo;
  private long myStartTimeNs;
  private long myEndTimeNs;
  private final FeatureTracker myFeatureTracker;
  private volatile boolean myIsDoneLoading = false;
  private volatile boolean myIsLoadingError = false;
  // Allocation records do not have heap information, but we create a fake HeapSet container anyway so that we have a consistent MemoryObject model.
  private final HeapSet myFakeHeapSet;

  public LegacyAllocationCaptureObject(@NotNull MemoryServiceBlockingStub client,
                                       @NotNull Common.Session session,
                                       @NotNull MemoryProfiler.AllocationsInfo info,
                                       @NotNull FeatureTracker featureTracker) {
    myClient = client;
    myClassDb = new ClassDb();
    mySession = session;
    myInfo = info;
    myStartTimeNs = info.getStartTime();
    myEndTimeNs = info.getEndTime();
    myFakeHeapSet = new HeapSet(this, DEFAULT_HEAP_NAME, DEFAULT_HEAP_ID);
    myFeatureTracker = featureTracker;
  }

  @NotNull
  @Override
  public String getName() {
    return "Recorded Allocations";
  }

  @Override
  public boolean isExportable() {
    return true;
  }

  @Nullable
  @Override
  public String getExportableExtension() {
    return "alloc";
  }

  @Override
  public void saveToFile(@NotNull OutputStream outputStream) {
    saveLegacyAllocationToFile(myClient, mySession, myInfo, outputStream, myFeatureTracker);
  }

  @NotNull
  @Override
  public Stream<InstanceObject> getInstances() {
    //noinspection ConstantConditions
    assert isDoneLoading() && !isError();
    return myFakeHeapSet.getInstancesStream();
  }

  @Override
  public long getStartTimeNs() {
    return myStartTimeNs;
  }

  @Override
  public long getEndTimeNs() {
    return myEndTimeNs;
  }

  @Override
  public boolean load(@Nullable Range queryRange, @Nullable Executor queryJoiner) {
    MemoryProfiler.LegacyAllocationEventsResponse response;
    while (true) {
      response = myClient.getLegacyAllocationEvents(MemoryProfiler.LegacyAllocationEventsRequest.newBuilder()
                                                      .setSession(mySession)
                                                      .setStartTime(myStartTimeNs)
                                                      .setEndTime(myEndTimeNs).build());
      if (response.getStatus() == MemoryProfiler.LegacyAllocationEventsResponse.Status.SUCCESS) {
        break;
      }
      else if (response.getStatus() == MemoryProfiler.LegacyAllocationEventsResponse.Status.NOT_READY) {
        try {
          Thread.sleep(50L);
        }
        catch (InterruptedException e) {
          Thread.currentThread().interrupt();
          myIsLoadingError = true;
          return false;
        }
        continue;
      }
      myIsLoadingError = true;
      return false;
    }

    MemoryProfiler.LegacyAllocationContextsRequest contextRequest = MemoryProfiler.LegacyAllocationContextsRequest.newBuilder()
      .setSession(mySession)
      .addAllStackIds(response.getEventsList().stream().map(LegacyAllocationEvent::getStackId).collect(Collectors.toSet()))
      .addAllClassIds(response.getEventsList().stream().map(LegacyAllocationEvent::getClassId).collect(Collectors.toSet()))
      .build();
    MemoryProfiler.AllocationContextsResponse contextsResponse = myClient.getLegacyAllocationContexts(contextRequest);

    // TODO remove this map, since we have built-in functionality in ClassDb now.
    Map<Integer, ClassDb.ClassEntry> classEntryMap = new HashMap<>();
    Map<Integer, MemoryProfiler.AllocationStack> callStacks = new HashMap<>();
    contextsResponse.getAllocatedClassesList().forEach(
      className -> classEntryMap.put(className.getClassId(), myClassDb.registerClass(DEFAULT_CLASSLOADER_ID, className.getClassName())));
    contextsResponse.getAllocationStacksList().forEach(callStack -> callStacks.putIfAbsent(callStack.getStackId(), callStack));

    // TODO make sure class IDs fall into a global pool
    for (LegacyAllocationEvent event : response.getEventsList()) {
      assert classEntryMap.containsKey(event.getClassId());
      assert callStacks.containsKey(event.getStackId());
      myFakeHeapSet.addDeltaInstanceObject(
        new LegacyAllocationsInstanceObject(event, classEntryMap.get(event.getClassId()), callStacks.get(event.getStackId())));
    }
    myIsDoneLoading = true;

    return true;
  }

  @Override
  public boolean isDoneLoading() {
    return myIsDoneLoading || myIsLoadingError;
  }

  @Override
  public boolean isError() {
    return myIsLoadingError;
  }

  @Override
  public void unload() {

  }

  @Override
  @NotNull
  public List<ClassifierAttribute> getClassifierAttributes() {
    return Arrays.asList(ClassifierAttribute.LABEL, ClassifierAttribute.ALLOCATIONS, ClassifierAttribute.SHALLOW_SIZE);
  }

  @NotNull
  @Override
  public List<CaptureObject.InstanceAttribute> getInstanceAttributes() {
    return Arrays.asList(CaptureObject.InstanceAttribute.LABEL, CaptureObject.InstanceAttribute.SHALLOW_SIZE);
  }

  @NotNull
  @Override
  public Collection<HeapSet> getHeapSets() {
    return Collections.singletonList(myFakeHeapSet);
  }

  @Override
  @Nullable
  public HeapSet getHeapSet(int heapId) {
    assert heapId == DEFAULT_HEAP_ID;
    return myFakeHeapSet;
  }
}

==> ideaIC-201.6668.113-sources.jar: com/android/tools/profilers/memory/adapters/LegacyAllocationCaptureObject.java <==
/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.profilers.memory.adapters;

import static com.android.tools.profilers.memory.MemoryProfiler.saveLegacyAllocationToFile;

import com.android.tools.adtui.model.Range;
import com.android.tools.profiler.proto.Common;
import com.android.tools.profiler.proto.Memory;
import com.android.tools.profiler.proto.Transport;
import com.android.tools.profilers.ProfilerClient;
import com.android.tools.profilers.analytics.FeatureTracker;
import com.android.tools.profilers.memory.LegacyAllocationConverter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.OutputStream;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.stream.Stream;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public final class LegacyAllocationCaptureObject implements CaptureObject {
  static final int DEFAULT_HEAP_ID = 0;
  static final String DEFAULT_HEAP_NAME = "default";

  @NotNull private final ProfilerClient myClient;
  @NotNull private final ClassDb myClassDb;
  @NotNull private final Common.Session mySession;
  @NotNull private final Memory.AllocationsInfo myInfo;
  @NotNull private final LegacyAllocationConverter myAllocationConverter;
  private long myStartTimeNs;
  private long myEndTimeNs;
  private final FeatureTracker myFeatureTracker;
  private volatile boolean myIsDoneLoading = false;
  private volatile boolean myIsLoadingError = false;
  // Allocation records do not have heap information, but we create a fake HeapSet container anyway so that we have a consistent MemoryObject model.
  private final HeapSet myFakeHeapSet;

  public LegacyAllocationCaptureObject(@NotNull ProfilerClient client,
                                       @NotNull Common.Session session,
                                       @NotNull Memory.AllocationsInfo info,
                                       @NotNull FeatureTracker featureTracker) {
    myClient = client;
    myClassDb = new ClassDb();
    mySession = session;
    myInfo = info;
    myAllocationConverter = new LegacyAllocationConverter();
    myStartTimeNs = info.getStartTime();
    myEndTimeNs = info.getEndTime();
    myFakeHeapSet = new HeapSet(this, DEFAULT_HEAP_NAME, DEFAULT_HEAP_ID);
    myFeatureTracker = featureTracker;
  }

  @NotNull
  @Override
  public String getName() {
    return "Recorded Allocations";
  }

  @Override
  public boolean isExportable() {
    return true;
  }

  @Nullable
  @Override
  public String getExportableExtension() {
    return "alloc";
  }

  @Override
  public void saveToFile(@NotNull OutputStream outputStream) {
    saveLegacyAllocationToFile(myClient, mySession, myInfo, outputStream, myFeatureTracker);
  }

  @NotNull
  @Override
  public Stream<InstanceObject> getInstances() {
    //noinspection ConstantConditions
    assert isDoneLoading() && !isError();
    return myFakeHeapSet.getInstancesStream();
  }

  @Override
  public long getStartTimeNs() {
    return myStartTimeNs;
  }

  @Override
  public long getEndTimeNs() {
    return myEndTimeNs;
  }

  @NotNull
  @Override
  public ClassDb getClassDatabase() {
    return myClassDb;
  }

  @Override
  public boolean load(@Nullable Range queryRange, @Nullable Executor queryJoiner) {
    if (!myInfo.getSuccess()) {
      myIsLoadingError = true;
      return false;
    }

    Transport.BytesResponse response;
    while (true) {
      response = myClient.getTransportClient().getBytes(Transport.BytesRequest.newBuilder()
                                                          .setStreamId(mySession.getStreamId())
                                                          .setId(Long.toString(myInfo.getStartTime()))
                                                          .build());
      if (!response.getContents().isEmpty()) {
        break;
      }
      else {
        try {
          Thread.sleep(50L);
        }
        catch (InterruptedException e) {
          Thread.currentThread().interrupt();
          myIsLoadingError = true;
          return false;
        }
      }
    }

    // TODO remove this map, since we have built-in functionality in ClassDb now.
    Map<Integer, ClassDb.ClassEntry> classEntryMap = new HashMap<>();
    Map<Integer, Memory.AllocationStack> callStacks = new HashMap<>();
    myAllocationConverter.parseDump(response.getContents().toByteArray());
    myAllocationConverter.getAllocationStacks().forEach(stack -> callStacks.putIfAbsent(stack.getStackId(), stack));
    myAllocationConverter.getClassNames().forEach(
      // We don't have super class information so just assign invalid id as the super class id.
      klass -> classEntryMap.put(klass.getClassId(), myClassDb.registerClass(klass.getClassId(), klass.getClassName())));

    for (Memory.AllocationEvent.Allocation event : myAllocationConverter.getAllocationEvents()) {
      assert classEntryMap.containsKey(event.getClassTag());
      assert callStacks.containsKey(event.getStackId());
      myFakeHeapSet.addDeltaInstanceObject(
        new LegacyAllocationsInstanceObject(event, classEntryMap.get(event.getClassTag()), callStacks.get(event.getStackId())));
    }
    myIsDoneLoading = true;

    return true;
  }

  @Override
  public boolean isDoneLoading() {
    return myIsDoneLoading || myIsLoadingError;
  }

  @Override
  public boolean isError() {
    return myIsLoadingError;
  }

  @Override
  public void unload() {

  }

  @Override
  @NotNull
  public List<ClassifierAttribute> getClassifierAttributes() {
    return Arrays.asList(ClassifierAttribute.LABEL, ClassifierAttribute.ALLOCATIONS, ClassifierAttribute.SHALLOW_SIZE);
  }

  @NotNull
  @Override
  public List<CaptureObject.InstanceAttribute> getInstanceAttributes() {
    return Arrays.asList(CaptureObject.InstanceAttribute.LABEL, CaptureObject.InstanceAttribute.SHALLOW_SIZE);
  }

  @NotNull
  @Override
  public Collection<HeapSet> getHeapSets() {
    return Collections.singletonList(myFakeHeapSet);
  }

  @Override
  @Nullable
  public HeapSet getHeapSet(int heapId) {
    assert heapId == DEFAULT_HEAP_ID;
    return myFakeHeapSet;
  }
}
